from fabtastic.fabric.commands.c_common import *
from fabtastic.fabric.commands.c_git import *
from fabtastic.fabric.commands.c_s3cmd import *

try:
    import mediasync
    from fabtastic.fabric.commands.c_mediasync import *
except ImportError:
    pass

try:
    import south
    from fabtastic.fabric.commands.c_south import *
except ImportError:
    pass

try:
    import gunicorn
    from fabtastic.fabric.commands.c_gunicorn import *
except ImportError:
    pass

try:
    import celery
    from fabtastic.fabric.commands.c_celery import *
except ImportError:
    pass

# Used to prevent double migrations.
env.already_db_migrated = False
env.already_media_synced = False


from fabric.api import *
from fabtastic.fabric.util import _current_host_has_role

def git_pull(roles='webapp_servers'):
    """
    Pulls the latest master branch from the git repo.
    """
    if _current_host_has_role(roles):
        print("=== PULLING FROM GIT ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("git pull")
            # Remove .pyc files for modules that no longer exist.
            run("find . -name '*.pyc' -delete")



from subprocess import call
from django.core.management.base import BaseCommand
from fabric.api import *
import fabfile

class Command(BaseCommand):
    help = "Updates your virtualenv from requirements.txt."

    def handle(self, *args, **options):
        pip_req_path = env.PIP_REQUIREMENTS_PATH
        cmd = ['pip', 'install', '--upgrade', '-r', pip_req_path]
        call(cmd)        



from fabric.api import *

def _current_host_has_role(role_str):
    """
    Looks to see if the host the current task is being executed on has
    the specified role.
    """
    if len(env.roledefs) is 0 and env.hosts:
        # No roledefs defined, but env.hosts is. If we set env.hosts, assume
        # that the operation should be done to everything in env.hosts.
        return True
    
    # Otherwise check the role list for the current host in env.
    return env['host_string'] in env.roledefs[role_str] 


import os
import stat
from bz2 import BZ2File
from subprocess import Popen, PIPE, call

def set_pgpass(database):
    """
    Sets the ~/.pgpass file up so that psql and pg_dump doesn't ask 
    for a password.
    """
    pgpass_file = os.path.expanduser('~')
    pgpass_file = os.path.join(pgpass_file, '.pgpass')
    
    db_host = database['HOST']
    db_port = database['PORT']
    #db_name = database['NAME']
    # Wildcarded for dropdb.
    db_name = '*'
    db_user = database['USER']
    db_pass = database['PASSWORD']
    
    if db_host is '':
        db_host = '*'
    if db_port is '':
        db_port = '*'
    
    fd = open(pgpass_file, 'wb')
    # host:port:database:username:password
    fd.write('%s:%s:%s:%s:%s' % (db_host, db_port, db_name, db_user, db_pass))
    fd.close()
    
    perms = stat.S_IRUSR | stat.S_IWUSR
    os.chmod(pgpass_file, perms)
    
def add_common_options_to_cmd(cmd, database, no_password_prompt=False,
                              **kwargs):
    """
    Adds some commonly used options to the given command string. psql,
    pg_dump, pg_restore, and a few others have some flags in common.
    
    cmd: (list) A command list, in format for Popen/call().
    """
    if database['HOST'] is not '':
        cmd.append('--host=%s' % database['HOST'])
        
    if database['PORT'] is not '':
        cmd.append('--port=%s' % database['PORT'])
        
    if no_password_prompt:
        cmd.append('--no-password')
        
    cmd.append('--username=%s' % database['USER'])

def dump_db_to_file(dump_path, database, no_owner=True, **kwargs):
    """
    pg_dumps the specified database to a given path. Passes output through
    bzip2 for compression.
    
    dump_path: (str) Complete path (with filename) to pg_dump out to.
    database: (dict) Django 1.2 style DATABASE settings dict.
    no_password_prompt: (bool) When True, never prompt for password, and fail
                               if none is provided by env variables or .pgpass.
                               
    Returns the path that was dumped to.
    """
    # Set a .pgpass file up so we're not prompted for a password.
    set_pgpass(database)
    
    cmd = ['pg_dump', '-i']
    
    # Add some common postgres options.
    add_common_options_to_cmd(cmd, database, **kwargs)
    
    # Plain formatting
    cmd.append('--format=p')

    if no_owner:
        cmd.append('--no-owner')

    cmd.append(database['NAME'])
    
    print "pg_dumping database '%s' to %s" % (database['NAME'], dump_path)
    
    # Run pg_dump
    db_dump = Popen(cmd, stdout=PIPE)
    # Open the eventual .tar.bz2 file for writing by bzip2.
    tfile = open(dump_path, 'w')
    # Use bzip2 to dump into the open file handle via stdout.
    db_bzip = Popen(['bzip2'], stdin=db_dump.stdout, stdout=tfile)
    db_bzip.wait()
    tfile.close()
    
    print "Database dump complete."
    
    return dump_path

def drop_db(database, **kwargs):
    """
    Drops the specified database.
    """
    # Set a .pgpass file up so we're not prompted for a password.
    set_pgpass(database)
    
    cmd = ['dropdb']
    
    # Add some common postgres options.
    add_common_options_to_cmd(cmd, database, **kwargs)
    
    cmd.append(database['NAME'])
    
    call(cmd)
    
def create_db(database, **kwargs):
    """
    Creates the specified database.
    """
    # Set a .pgpass file up so we're not prompted for a password.
    set_pgpass(database)
    
    cmd = ['createdb']
    
    add_common_options_to_cmd(cmd, database, no_password_prompt=False)
    
    cmd.append('--owner=%s' % database['USER'])
    cmd.append(database['NAME'])
    call(cmd)

def restore_db_from_file(dump_path, database, **kwargs):
    """
    Restores the specified database from a pg_dump file.
    
    dump_path: (str) Complete path (with filename) to pg_restore from.
    database: (dict) Django 1.2 style DATABASE settings dict.
    """
    decompress = ['bunzip2', '--keep', dump_path]
    print "De-compressing %s" % dump_path
    call(decompress)
    
    # Yank the .bz2 off of the end of the dump_path, now that it's
    # decompressed.
    decompresed_path = dump_path[:-4]
    
    # Set a .pgpass file up so we're not prompted for a password.
    set_pgpass(database)
    
    cmd = ['psql', '-q']
    
    # Add some common postgres options.
    add_common_options_to_cmd(cmd, database, **kwargs)

    cmd.append('--dbname=%s' % database['NAME'])
    cmd.append('--file=%s' % decompresed_path)

    print "Running pg_restore"
    # Run the assembled pg_restore above.
    Popen(cmd).wait()
    
    # Get rid of the decompressed db dump.
    del_decompressed = ['rm', '-f', decompresed_path]
    call(del_decompressed)
    
    print "Restoration complete."


import os
import sys
from fabric.api import *
from fabtastic import db
from fabtastic.fabric.util import _current_host_has_role

def get_remote_db(roles='webapp_servers'):
    """
    Retrieves a remote DB dump and dumps it in your project's root directory.
    """
    if _current_host_has_role(roles):
        dump_filename = db.util.get_db_dump_filename()
        dump_path = os.path.join(env.REMOTE_CODEBASE_PATH, dump_filename)

        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py ft_dump_db %s" % (
                env.REMOTE_VIRTUALENV_NAME,
                dump_filename))
            get(dump_path, dump_filename)
            run("rm %s" % dump_filename)

        # In a multi-host environment, target hostname is appended by Fabric.
        # TODO: Make this use Fabric 1.0's improved get() when it's released.
        new_filename = '%s.%s' % (dump_filename, env['host'])
        # Move it back to what it should be.
        local('mv %s %s' % (new_filename, dump_filename))

        # Die after this to prevent executing this with more hosts.
        sys.exit(0)

def sync_to_remote_db(roles='webapp_servers'):
    """
    Retrieves a remote DB dump, wipes your local DB, and installs the
    remote copy in place.
    """
    if _current_host_has_role(roles):
        dump_filename = db.util.get_db_dump_filename()
        dump_path = os.path.join(env.REMOTE_CODEBASE_PATH, dump_filename)

        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py ft_dump_db %s" % (
                env.REMOTE_VIRTUALENV_NAME,
                dump_filename))
            get(dump_path, dump_filename)
            run("rm %s" % dump_filename)

        # In a multi-host environment, target hostname is appended by Fabric. 
        # TODO: Make this use Fabric 1.0's improved get() when it's released.
        filename_with_hostname = '%s.%s' % (dump_filename, env['host'])
        if os.path.exists(filename_with_hostname):
            # Move it back to what it should be.
            local('mv %s %s' % (filename_with_hostname, dump_filename))
        local('./manage.py ft_restore_db %s' % dump_filename, capture=False)
        local('rm %s' % dump_filename)

        # Die after this to prevent executing this with more hosts.
        sys.exit(0)

def flush_cache(roles='webapp_servers'):
    """
    Flushes the cache.
    """
    if _current_host_has_role(roles):
        print("=== FLUSHING CACHE ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py ft_clear_cache" % env.REMOTE_VIRTUALENV_NAME)

def pip_update_reqs(roles='webapp_servers'):
    """
    Updates your virtualenv from requirements.txt.
    """
    if _current_host_has_role(roles):
        print("=== UPDATING REQUIREMENTS ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py ft_pip_update_reqs" % env.REMOTE_VIRTUALENV_NAME)

def fabtastic_update(roles='webapp_servers'):
    """
    Updates your copy of django-fabtastic from the git repository.
    """
    if _current_host_has_role(roles):
        print("=== UPDATING FABTASTIC ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py ft_fabtastic_update" % env.REMOTE_VIRTUALENV_NAME)



import os
from subprocess import call

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from fabric.api import *
from fabtastic import db
import fabfile

class Command(BaseCommand):
    help = 'Backs the DB up to S3. Make sure to run s3cmd --configure.'

    def handle(self, *args, **options):
        # Get DB settings from settings.py.
        database = db.util.get_db_setting_dict()
        # Generate a temporary DB dump filename.      
        dump_filename = db.util.get_db_dump_filename()
        # Carry out the DB dump.
        db.dump_db_to_file(dump_filename, database)
        
        # Now upload via s3cmd. See note above about s3cmd --configure.
        cmd = ['s3cmd', 'put']
        cmd.append(dump_filename)
        cmd.append('s3://%s/%s' % (env.S3_DB_BACKUP_BUCKET, dump_filename))
        call(cmd)
        
        # Clean up the temporary download file.
        os.remove(dump_filename)



import os
from django.core.management.base import BaseCommand, CommandError
from django.contrib.contenttypes.models import ContentType
from fabtastic import db

class Command(BaseCommand):
    args = '[<output_file_path>]'
    help = 'Dumps a SQL backup of your entire DB. Defaults to CWD.'
                        
    def get_dump_path(self):
        """
        Determines the path to write the SQL dump to. Depends on whether the
        user specified a path or not.
        """
        if len(self.args) > 0:
            return self.args[0]
        else:
            dump_filename = db.util.get_db_dump_filename()
            return os.path.join(os.getcwd(), dump_filename)
        
    def handle(self, *args, **options):
        """
        Handle raw input.
        """
        self.args = args
        self.options = options
        
        # Get DB settings from settings.py.
        database = db.util.get_db_setting_dict()
        # Figure out where to dump the file to.
        dump_path = self.get_dump_path()

        # Run the db dump.
        db.dump_db_to_file(dump_path, database)


"""
Various database-related utility functions.
"""
import datetime
from django.conf import settings

def get_db_setting(db_setting, db_alias='default'):
    """
    Gets a database setting from settings.py. Extra logic to support
    Django 1.1 and Django 1.2.
    
    db_setting: (str) One of the database setting names (with key values
                      matching Django 1.2). For example, 'name', 'port', 'host'.
    db_alias: (str) In the case of settings in Django 1.2 format, get settings
                    for a DB other than the default.
    """
    if hasattr(settings, 'DATABASES'):
        return settings.DATABASES[db_alias][db_setting]
    else:
        return getattr(settings, 'DATABASE_%s' % db_setting.upper())
    
def get_db_setting_dict(db_alias='default'):
    """
    Returns a dict of DB settings, as per the Django 1.2 DATABASE settings.py
    dict. This can be used as a compatibility measure for Django 1.1 and
    earlier.
    """
    return {
        'ENGINE': get_db_setting('ENGINE', db_alias),
        'NAME': get_db_setting('NAME', db_alias),
        'HOST': get_db_setting('HOST', db_alias),
        'PORT': get_db_setting('PORT', db_alias),
        'USER': get_db_setting('USER', db_alias),
        'PASSWORD': get_db_setting('PASSWORD', db_alias),
    }
    
def get_db_dump_filename(db_alias='default'):
    """
    Returns a generic DB dump file name for use with backup/restore scripts.
    Fabric commands share this function with the management commands.
    """
    today = datetime.datetime.today()
    database_dict = get_db_setting_dict(db_alias)

    dump_filename = "%s-%s.sql.tar.bz2" %  (
        database_dict['NAME'], 
        today.strftime("%Y_%m_%d-%H%M"),
    )
    return dump_filename


import sys
from fabric.api import *
from fabtastic.fabric.util import _current_host_has_role
       
def backup_db_to_s3():
    """
    Backs up the DB to Amazon S3. The DB server runs pg_dump,
    then uploads to S3 via the s3cmd command. On new DB instances, you'll need
    to run 's3cmd --configure' (as the user that will be running s3cmd) to setup 
    the keys. You'll notice they aren't passed here as a result of that.
    """
    if _current_host_has_role('webapp_servers'):        
        print("=== BACKING UP DB TO S3 ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py ft_backup_db_to_s3" % env.REMOTE_VIRTUALENV_NAME)
        print("DB backed up to S3.")
        
        # Die after this to prevent executing this with more hosts.
        sys.exit(0)


from django.conf import settings
from fabtastic.db import util

db_engine = util.get_db_setting('ENGINE')

if 'postgresql_psycopg2' in db_engine:
    from fabtastic.db.postgres import *
else:
    raise NotImplementedError("Fabtastic: DB engine '%s' is not supported" % db_engine)


from fabric.api import *
from fabtastic.fabric.util import _current_host_has_role

def mediasync_syncmedia(roles='webapp_servers'):
    """
    Syncs the checked out git media with S3.
    """
    if _current_host_has_role(roles) and not env.already_media_synced:
        print("=== SYNCING STATIC MEDIA WITH S3 ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py syncmedia" % env.REMOTE_VIRTUALENV_NAME)
        env.already_media_synced = True


from django.core.management.base import BaseCommand, CommandError

class Command(BaseCommand):
    help = 'Restarts all celery daemons.'

    def handle(self, *args, **options):
        try:
            from celery.task.control import broadcast
        except ImportError:
            raise CommandError("Celery is not currently installed.")
        
        # Shut them all down.
        broadcast("shutdown")


from django.core.management.base import BaseCommand
from fabtastic.util.cache import clear_django_cache

class Command(BaseCommand):
    help = "Flushes the Django cache."

    def handle(self, *args, **options):
        clear_django_cache()            



from fabric.api import *
from fabtastic.fabric.util import _current_host_has_role

def gunicorn_restart_workers():
    """
    Reloads gunicorn. This must be done to re-compile the code after a new
    revision has been checked out.
    """
    if _current_host_has_role('webapp_servers'):
        print("=== RESTARTING GUNICORN WORKERS ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py ft_gunicorn_restart" % env.REMOTE_VIRTUALENV_NAME)
        print("Gunicorn reloaded")


from fabric.api import *
from fabtastic.fabric.util import _current_host_has_role

def south_migrate():
    """
    Migrates the DB schema with South. Sets already_db_migrated to prevent
    double migrations.
    """
    if _current_host_has_role('webapp_servers') and not env.already_db_migrated:
        print("=== RUNNING SOUTH DB MIGRATIONS ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("workon %s && ./manage.py migrate" % env.REMOTE_VIRTUALENV_NAME)
        env.already_db_migrated = True


import os
from datetime import datetime
from django.core.management.base import BaseCommand, CommandError
from django.contrib.contenttypes.models import ContentType
from fabtastic import db

class Command(BaseCommand):
    args = '[<output_file_path>]'
    help = 'Restores a DB from a SQL dump file.'
                               
    def handle(self, *args, **options):
        """
        Handle raw input.
        """
        self.args = args
        self.options = options
        
        if len(self.args) < 1:
            raise CommandError('You must specify the path to the DB dump file to restore from.')

        # Path to file to restore from.
        dump_path = self.args[0]
        
        # Get DB settings from settings.py.
        database_dict = db.util.get_db_setting_dict()

        # Drop the DB.
        db.drop_db(database_dict)
        # Re-create an empty DB with the same name.
        db.create_db(database_dict)
        # Restore from the DB dump.
        db.restore_db_from_file(dump_path, database_dict)


"""
Various cache-related general utility functions.
"""
from django.core.cache import cache

def clear_django_cache():
    """
    Clears the Django cache through the cache backend.
    """
    try:
        cache._cache.flush_all()
    except AttributeError:
        # try filesystem caching next
        old = cache._cull_frequency
        old_max = cache._max_entries
        cache._max_entries = 0
        cache._cull_frequency = 1
        cache._cull()
        cache._cull_frequency = old
        cache._max_entries = old_max


from distutils.core import setup
import fabtastic

long_description = open('README.rst').read()

setup(
    name='django-fabtastic',
    version=fabtastic.VERSION,
    packages=['fabtastic', 
              'fabtastic.db', 
              'fabtastic.fabric',
              'fabtastic.fabric.commands', 
              'fabtastic.management',
              'fabtastic.management.commands',
              'fabtastic.util'],
    description='A Django app that uses Django manage.py extension commands with Fabric for deployment.',
    long_description=long_description,
    author='Gregory Taylor',
    author_email='gtaylor@duointeractive.com',
    license='BSD License',
    url='http://github.com/duointeractive/django-fabtastic',
    platforms=["any"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)


from subprocess import call
from django.core.management.base import BaseCommand

class Command(BaseCommand):
    help = "Updates your copy of Fabtastic to the latest from git."

    def handle(self, *args, **options):
        fabtastic_repo = 'git+http://github.com/duointeractive/django-fabtastic.git#egg=fabtastic'
        cmd = ['pip', 'install', '--upgrade', fabtastic_repo]
        call(cmd)        



from fabric.api import *
from fabtastic.fabric.util import _current_host_has_role

def celeryd_restart(roles='webapp_servers'):
    """
    Reloads celeryd. This must be done to re-compile the code after a new
    revision has been checked out.
    
    NOTE: This broadcasts a 'shutdown' call to all celery workers. You must have
    supervisor or something running to start them back up, or this ends up
    just being a shutdown (sans restart).
    """
    if _current_host_has_role(roles):
        print("=== RESTARTING CELERY DAEMON ===")
        with cd(env.REMOTE_CODEBASE_PATH):
            run("./manage.py ft_celeryd_restart")
            print "Celery shutdown broadcasted, workers restarting."


import os
import signal

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from fabric.api import *
import fabfile

class Command(BaseCommand):
    help = 'Restarts gunicorn workers after code changes.'

    def handle(self, *args, **options):
        pid_path = env.GUNICORN_PID_PATH
        if os.path.exists(pid_path):
            pid = int(open(pid_path, 'r').read())
            os.kill(pid, signal.SIGHUP)
        else:
            raise CommandError("No gunicorn process running.")